from flask_socketio import emit
from mxcubeweb.server import Server as server
from mxcubeweb.app import MXCUBEApplication as mxcube

import json


def flush():
    mxcube.UI_STATE = dict()


def init():
    @server.flask_socketio.on("connect", namespace="/ui_state")
    @server.ws_restrict
    def connect():
        pass

    @server.flask_socketio.on("disconnect", namespace="/ui_state")
    def disconnect():
        pass

    @server.flask_socketio.on("ui_state_get", namespace="/ui_state")
    def ui_state_get(k):
        k = k.replace("reduxPersist:", "")
        # print 'ui state GET',k,'returning:',STATE[k]
        return json.dumps(mxcube.UI_STATE[k])

    @server.flask_socketio.on("ui_state_rm", namespace="/ui_state")
    def ui_state_rm(k):
        k = k.replace("reduxPersist:", "")
        # print 'ui state REMOVE',k
        del mxcube.UI_STATE[k]

    @server.flask_socketio.on("ui_state_set", namespace="/ui_state")
    def ui_state_update(key_val):
        key, val = key_val
        mxcube.UI_STATE[key.replace("reduxPersist:", "")] = json.loads(val)

        emit(
            "state_update",
            json.dumps(mxcube.UI_STATE),
            namespace="/ui_state",
            room="observers",
            include_self=False,
        )

    @server.flask_socketio.on("ui_state_getkeys", namespace="/ui_state")
    def ui_state_getkeys(*args):
        return ["reduxPersist:" + k for k in mxcube.UI_STATE.keys()]



from mxcubeweb.core.adapter.adapter_base import AdapterBase


class DetectorAdapter(AdapterBase):
    def __init__(self, ho, *args):
        """
        Args:
            (object): Hardware object.
        """
        super(DetectorAdapter, self).__init__(ho, *args)
        ho.connect("stateChanged", self._state_change)

    def _state_change(self, *args, **kwargs):
        self.state_change(*args, **kwargs)

    def state(self):
        return self._ho.get_state().name.upper()



from mxcubeweb.core.adapter.adapter_base import ActuatorAdapterBase
from mxcubeweb.core.util.adapterutils import export
from mxcubeweb.core.util.networkutils import RateLimited

from mxcubeweb.core.models.adaptermodels import (
    HOActuatorValueChangeModel,
    FloatValueModel,
)


class ActuatorAdapter(ActuatorAdapterBase):
    """
    Adapter for Energy Hardware Object, a web socket is used to communicate
    information on longer running processes.
    """

    def __init__(self, ho, *args):
        """
        Args:
            (object): Hardware object.
        """
        super(ActuatorAdapter, self).__init__(ho, *args)
        self._event_rate = 4

        @RateLimited(self._event_rate)
        def _vc(value, **kwargs):
            self.value_change(value, **kwargs)

        self._vc = _vc

        try:
            ho.connect("valueChanged", self._value_change)
            ho.connect("stateChanged", self.state_change)
        except Exception:
            pass

    def _value_change(self, *args, **kwargs):
        self._vc(*args, **kwargs)

    def _set_value(self, value: HOActuatorValueChangeModel):
        """
        Execute the sequence to set the value.
        Args:
            value (float): Target energy [keV].
        Returns:
            (float as str): The actual value set.
        Raises:
            ValueError: Value not valid or attemp to set a non-tunable energy.
            RuntimeError: Timeout while setting the value.
            StopItteration: When a value change was interrupted (abort/cancel).
        """
        try:
            self._ho.set_value(float(value.value))
        except Exception:
            raise

    @export
    def _get_value(self) -> FloatValueModel:
        """
        Read the energy.
        Returns:
            (float as str): Energy [keV].
        Raises:
            ValueError: When value for any reason can't be retrieved.
        """
        try:
            return FloatValueModel(**{"value": self._ho.get_value()})
        except (AttributeError, TypeError):
            raise ValueError("Could not get value")

    def stop(self):
        """
        Stop the execution.
        """
        self._ho.abort()

    def read_only(self):
        """
        Check if the energy is tunable or not.
        Retuns:
            (bool): True if tunable, False if not.
        """
        return self._ho.read_only
